%%%%%%%%%%%%%%
% THR_TEST.M %
%%%%%%%%%%%%%%

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/


% This is a GAUSS procedure.
% It computes a test for a threshold in linear regression
% under homoskedasticity.  The procedure takes the form

% [f_test,p_value] = thr_test(dat,yi,xi,qi,trim_per,rep)

% The inputs are:
% dat      = data matrix (nxk)
% yi       = index of dependent (y) variable, e.g. yi = 1
% xi       = indexes of independent (x) variables, e.g. xi = [2;3]
% qi       = index of threshold (q) variable, e.g. qi = 4
% trim_per = percentage of sample to trim from ends, e.g. trim_per = .15
% rep      = number of bootstrap replications, e.g. rep = 1000

% Output:
% f_test   = Value of Maximal (Quandt) F-statistic
% p_value  = Bootstrap p-value

% Notes: 
% (1)    Do not include a constant in the independent variables,
%        the program automatically adds an intercept to the regression.

% (2)    The program stores the sequential design matrices in order
%        to speed up the bootstrap computations.  It is possible that if
%        your dataset is very large, this will exceed your computer RAM
%        memory.  If so, program will crash, and the message
%               error: Insufficient Workspace Memory
%        will be displayed.  If a more RAM is not available, 
%        change the switch quick_ in the program to read "quick_=0;".  
%        This switch requires the bootstrap to re-calculate the design
%        matrices for each bootstrap replication, which requires less memory,
%        but somewhat more computer time.

% Example:
% If the nxk matrix "dat" contains the dependent variable in the first
% column, the independent variables in the second through tenth columns,
% and the threshold variable in the fifth:

% xi = [2;3;4;5;6;7;8;9;10];
% [f_test,p_value] = thr_test(dat,1,xi,5,.15,1000);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Control Parameters, may be changed  */

cr_ = .95;        % This is the confidence level used to plot the
                  % critical value in the graph.  It is not used
                  % elsewhere in the analysis.                          @
graph_   = 1;     % Graph indicator
                  % Set graph_=0 to not view the graph of the likelihood
                  % Set graph_=1 to view the graph of the likelihood    @
quick_   = 1;     % Indicator of method used for bootstrap
                  % Set quick_=1 for efficient, quick computation
                  % Set quick_=0 if memory is limited (perhaps for
                  % large data sets).                                   @ 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP 

%TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP TEMP 


function [ftest,pv]=thr_test(dat,yi,xi,qi,trim_per,rep);
n=length(dat(:,1));
q=dat(:,qi);

% Sorted index of q %
qs=0;
for i=1:length(q)
    for i_=1:length(q)
        temp=0;
        for j=1:length(q)
            if (q(i_)>q(j))|(i_>=j&q(i_)==q(j))
                temp=temp+1;
            end;
        end;
        if temp==i
            qs=[qs;i_];
    end;
    end;
end;
qs=qs(2:length(qs));
q=q(qs);

y=dat(qs,yi);
x=[ones(n,1),dat(qs,xi)];
k=length(x(1,:));
qs=unique(q);
qn=length(qs);
qq=zero(qn,1);
r=1;
while r<=qn
    qq(r)=sum(q==qs(r));
    r=r+1;
end;
cqq=cumsum(qq);
sq=(cqq>=floor(n*trim_per)).*(cqq<=floor(n*(1-trim_per)));

temp_ps=ps(1,:);
temp_cqq=cqq(1,:);
temp=0;
for i=1:length(sq)
    if sq(i)==1
        if temp==0
            temp_ps=qs(i,:);
            temp_cqq=cqq(i,:);
            temp=1;
        else
            temp_ps=[temp_ps;qs(i,:)];
            temp_cqq=[temp_cqq;cqq(i,:)];
        end;
    end;
end;
ps=temp_ps;
clear temp_ps;
clear temp_cqq;

qn=length(qs(:,1));

mi=inv(x'*x);
e=y-x*mi*(x'*y);
ee=e'*e;
xe=cumsum(x.*e);
sn=zeros(qn,1);

if quick_==1
    mmistore=zeros(k*(k+1)/2,qn);
    cqqb=1;
    mm=zeros(k,k);
    r=1;
    while r<=qn
        cqqr=cqq(r);
        mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
        sume=xe(cqqr,:)';
        mmi=inv(mm-mm*mi*mm);
        sn(r)=sume'*mmi*sume;
        cqqb=cqqr+1;
        
        indmmi=1;
        for i=1:length(mmi(:,1))
            ii=1;
            while ii<=i
                mmistore(indmmi,r)=mmi(i,ii);
                indmmi=indmmi+1;
                ii=ii+1;
            end;
        end;
        clear indmmi;
        clear ii;
        
        r=r+1;
    end;
    for i=1:length(sn) %maxindc%   
        if sn(i)==max(sn)
           si=i;
           break;
        end;
    end;
    smax=sn(si);
    qmax=qs(si);
    sig=(ee-smax)/n;
    lr=sn/sig;
    ftest=inv(sig)*smax;
    fboot=zeros(rep,1);
    j=1;
    while j<=rep
        y=randn(n,1);
        e=y-x*mi*(x'*y);
        ee=e'*e;
        xe=cumsum(x.*(e*ones(1,length(x(1,:)))));
        sn=zeros(qn,1);
        r=1;
        while r<=qn
           ind=1; % xpnd % 
           for ix=1:sqrt(2*length(mmistore(:,r)))
               for iy=1:ix
                   mmi(ix,iy)=mmistore(ind,r);
                   mmi(iy,ix)=mmistore(ind,r);
                   ind=ind+1;
               end;
           end;
           clear ix;
           clear iy;
           sume=xe(cqq(r),:)';
           sn(r)=sume'*mmi*sume;
           r=r+1;
        end;
        smax=max(sn)';
        sig=(ee-smax)/n;
        fboot(j)=smax/sig;
        j=j+1;
    end;
elseif quick_==0
    cqqb=1;
    mm=zeros(k,k);
    r=1;
    while r<=qn;
        cqqr=cqq(r);
        mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
        sume=xe(cqqr,:)';
        mmi=mm-mm*mi*mm;
        sn(r)=sume'*(inv(mmi)*sume);
        cqqb=cqqr+1;
        r=r+1;
    end;
    for i=1:length(sn) %maxindc%   
        if sn(i)==max(sn)
           si=i;
           break;
        end;
    end;
    smax=sn(si);
    qmax=qs(si);
    sig=(ee-smax)/n;
    lr=inv(sig)*sn;
    ftest=inv(sig)*smax;
    fboot=zeros(rep,1);
    j=1;
    while j<=rep
        y=randn(n,1);
        e=y-x*mi*(x'*y);
        ee=e'*e;
        xe=cumsum(x.*(e*ones(1,length(x(1,:)))));
        sn=zeros(qn,1);
        cqqb=1;
        mm=zeros(k,k);
        r=1;
        while r<=qn
            cqqr=cqq(r);
            mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
            mmi=mm-mm*mi*mm;
            sume=xe(cqq(r),:)';
            sn(r)=sume'*(inv(mmi)*sume);
            cqqb=cqqr+1;
            r=r+1;
        end;
        smax=max(sn);
        sig=(ee-smax)/n;
        fboot(j)=inv(sig)*smax;
        j=j+1;
    end;
end;
fboot=sortrows(fboot,1);
pv=mean(fboot>=ftest);
cr=fboot(round(rep*cr_));
    
        
    